#include <SDL/SDL.h>
#include "sdl_SDL.h"

#define BUFFER_SIZE 256
jobject g_surface = NULL;

jclass getSDL_Surface(JNIEnv *env){
  return (*env)->FindClass(env, "Lsdl/Surface;");
}

jclass getSDL_PixelFormat(JNIEnv* env){
  return (*env)->FindClass(env, "Lsdl/PixelFormat;");
}

jclass getSDL_Rect(JNIEnv *env){
  return (*env)->FindClass(env, "Lsdl/Rect;");
}

jclass getSDL_Palette(JNIEnv *env){
  return (*env)->FindClass(env, "Lsdl/Palette;");
}

jclass getSDL_Color(JNIEnv *env){
  return (*env)->FindClass(env, "Lsdl/Color;");
}

jclass getSDL_VideoInfo(JNIEnv *env){
  return (*env)->FindClass(env, "Lsdl/VideoInfo;");
}

jint getSDL_SurfacePointer(JNIEnv *env, jobject surface){
  jobject sdl_surface_obj = (*env)->GetObjectClass(env, surface);
  jclass sdl_surface = getSDL_Surface(env);
  jfieldID fld_pointer = (*env)->GetFieldID(env, sdl_surface, "pointer", "I");
  jint pointer = (*env)->GetIntField(env, surface, fld_pointer);
  return pointer;

}

jobject createObjectFromSDL_Rect(JNIEnv* env, jclass sdl_rect, SDL_Rect* rect){
  jmethodID cns = (*env)->GetMethodID(env, sdl_rect, "<init>", "()V");
  jobject sdl_rect_obj = (*env)->NewObject(env, sdl_rect, cns);
  
  jfieldID fld_x = (*env)->GetFieldID(env, sdl_rect, "x", "I");
  jfieldID fld_y = (*env)->GetFieldID(env, sdl_rect, "y", "I");
  jfieldID fld_w = (*env)->GetFieldID(env, sdl_rect, "w", "I");
  jfieldID fld_h = (*env)->GetFieldID(env, sdl_rect, "h", "I");

  (*env)->SetIntField(env, sdl_rect_obj, fld_x, rect->x);
  (*env)->SetIntField(env, sdl_rect_obj, fld_y, rect->y);
  (*env)->SetIntField(env, sdl_rect_obj, fld_w, rect->w);
  (*env)->SetIntField(env, sdl_rect_obj, fld_h, rect->h);
  return sdl_rect_obj;
}

jobject createObjectFromSDL_Color(JNIEnv *env, jclass sdl_color, SDL_Color* colors){
  jmethodID cns = (*env)->GetMethodID(env, sdl_color, "<init>", "()V");
  jobject sdl_color_obj = (*env)->NewObject(env, sdl_color, cns);
  
  jfieldID fld_r = (*env)->GetFieldID(env, sdl_color, "r","I");
  jfieldID fld_g = (*env)->GetFieldID(env, sdl_color, "g","I");
  jfieldID fld_b = (*env)->GetFieldID(env, sdl_color, "b","I");
  jfieldID fld_unused = (*env)->GetFieldID(env, sdl_color, "unused","I");

  (*env)->SetIntField(env, sdl_color_obj, fld_r, colors->r);
  (*env)->SetIntField(env, sdl_color_obj, fld_g, colors->g);
  (*env)->SetIntField(env, sdl_color_obj, fld_b, colors->b);
  (*env)->SetIntField(env, sdl_color_obj, fld_unused, colors->unused);

  return sdl_color_obj;
}

jobject createObjectFromSDL_Palette(JNIEnv *env, jclass sdl_palette, SDL_Palette* palette){
  jmethodID cns = (*env)->GetMethodID(env, sdl_palette, "<init>", "()V");
  jobject sdl_palette_obj = (*env)->NewObject(env, sdl_palette, cns);

  jfieldID fld_pointer = (*env)->GetFieldID(env, sdl_palette, "pointer", "I");
  jfieldID fld_ncolors = (*env)->GetFieldID(env, sdl_palette, "ncolors","I");
  jfieldID fld_colors = (*env)->GetFieldID(env, sdl_palette, "colors","Lsdl/Color;");
  jclass sdl_color = getSDL_Color(env);

  jobject sdl_color_obj = createObjectFromSDL_Color(env, sdl_color, palette->colors);
  (*env)->SetIntField(env, sdl_palette_obj, fld_pointer, (jint)palette);
  (*env)->SetObjectField(env, sdl_palette_obj, fld_colors, sdl_color_obj);
  (*env)->SetIntField(env, sdl_palette_obj, fld_ncolors, palette->ncolors);
  return sdl_palette_obj;
}

jobject createObjectFromSDL_PixelFormat(JNIEnv *env, jclass sdl_pixel_format, SDL_PixelFormat* format){
  jmethodID cns = (*env)->GetMethodID(env, sdl_pixel_format, "<init>", "()V");
  jobject sdl_pixel_format_obj = (*env)->NewObject(env, sdl_pixel_format, cns);
  
  jfieldID fld_pointer = (*env)->GetFieldID(env, sdl_pixel_format, "pointer", "I");
  jfieldID fld_palette = (*env)->GetFieldID(env, sdl_pixel_format, "palette", "Lsdl/Palette;");
  jfieldID fld_bitsperpixel = (*env)->GetFieldID(env, sdl_pixel_format, "BitsPerPixel","I");
  jfieldID fld_bytesperpixel = (*env)->GetFieldID(env, sdl_pixel_format, "BytesPerPixel","I");
  jfieldID fld_rloss = (*env)->GetFieldID(env, sdl_pixel_format, "Rloss","I");
  jfieldID fld_gloss = (*env)->GetFieldID(env, sdl_pixel_format, "Gloss","I");
  jfieldID fld_bloss = (*env)->GetFieldID(env, sdl_pixel_format, "Bloss","I");
  jfieldID fld_aloss = (*env)->GetFieldID(env, sdl_pixel_format, "Aloss","I");
  jfieldID fld_rshift = (*env)->GetFieldID(env, sdl_pixel_format, "Rshift","I");
  jfieldID fld_gshift = (*env)->GetFieldID(env, sdl_pixel_format, "Gshift","I");
  jfieldID fld_bshift = (*env)->GetFieldID(env, sdl_pixel_format, "Bshift","I");
  jfieldID fld_ashift = (*env)->GetFieldID(env, sdl_pixel_format, "Ashift","I");
  jfieldID fld_rmask = (*env)->GetFieldID(env, sdl_pixel_format, "Rmask","I");
  jfieldID fld_gmask = (*env)->GetFieldID(env, sdl_pixel_format, "Gmask","I");
  jfieldID fld_bmask = (*env)->GetFieldID(env, sdl_pixel_format, "Bmask","I");
  jfieldID fld_amask = (*env)->GetFieldID(env, sdl_pixel_format, "Amask","I");
  jfieldID fld_colorkey = (*env)->GetFieldID(env, sdl_pixel_format, "colorkey","I");
  jfieldID fld_alpha = (*env)->GetFieldID(env, sdl_pixel_format, "alpha","I");

  jobject sdl_palette_obj = NULL;
  if(format->BitsPerPixel <= 8){ //8ビット以下の時のみPaletteをセット
    jclass sdl_palette = getSDL_Palette(env);
    sdl_palette_obj = createObjectFromSDL_Palette(env, sdl_palette, format->palette);
    (*env)->SetObjectField(env, sdl_pixel_format_obj, fld_palette, sdl_palette_obj);
  }
  
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_pointer, (jint)format);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_bitsperpixel, format->BitsPerPixel);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_bytesperpixel, format->BytesPerPixel);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_rloss, format->Rloss);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_gloss, format->Gloss);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_bloss, format->Bloss);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_aloss, format->Aloss);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_rshift, format->Rshift);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_gshift, format->Gshift);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_bshift, format->Bshift);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_ashift, format->Ashift);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_rmask, format->Rmask);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_gmask, format->Gmask);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_bmask, format->Bmask);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_amask, format->Amask);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_colorkey, format->colorkey);
  (*env)->SetIntField(env, sdl_pixel_format_obj, fld_alpha, format->alpha);
  return sdl_pixel_format_obj;
}


jobject createObjectFromSDL_Surface(JNIEnv* env, jclass sdl_surface, SDL_Surface* surface){
  jmethodID cns = (*env)->GetMethodID(env, sdl_surface, "<init>", "()V");
  jobject sdl_surface_obj = (*env)->NewObject(env, sdl_surface, cns);
  
  jfieldID fld_pointer = (*env)->GetFieldID(env, sdl_surface, "pointer", "I");
  jfieldID fld_flags = (*env)->GetFieldID(env, sdl_surface, "flags", "I");
  jfieldID fld_w = (*env)->GetFieldID(env, sdl_surface, "w", "I");
  jfieldID fld_h = (*env)->GetFieldID(env, sdl_surface, "h", "I");
  jfieldID fld_pitch = (*env)->GetFieldID(env, sdl_surface, "pitch", "I");
  jfieldID fld_refcount = (*env)->GetFieldID(env, sdl_surface, "refcount", "I");
  //SDL_Rectの作成とセット
  //L完全なパッケージ名;で指定しなければならない。
  jfieldID fld_clip_rect = (*env)->GetFieldID(env, sdl_surface, "clip_rect", "Lsdl/Rect;");
  SDL_Rect* rect = &(surface->clip_rect);
  jclass sdl_rect = getSDL_Rect(env);
  jobject sdl_rect_obj = createObjectFromSDL_Rect(env, sdl_rect, rect);
  (*env)->SetObjectField(env, sdl_surface_obj, fld_clip_rect, sdl_rect_obj);

  //SDL_PixelFormatの生成とセット
  jfieldID fld_format = (*env)->GetFieldID(env, sdl_surface, "format", "Lsdl/PixelFormat;");
  SDL_PixelFormat* format = surface->format;
  jclass sdl_format = getSDL_PixelFormat(env);
  jobject sdl_pixel_format_obj = createObjectFromSDL_PixelFormat(env, sdl_format, format);
  (*env)->SetObjectField(env, sdl_surface_obj, fld_format, sdl_pixel_format_obj);
  
  //pixelsポインターはByteBufferオブジェクトにしてセット
  jfieldID fld_pixels = (*env)->GetFieldID(env, sdl_surface, "pixels", "Ljava/nio/ByteBuffer;");
  jlong pixels_size = surface->pitch * surface->h;
  jobject pixels_obj = (*env)->NewDirectByteBuffer(env, (void *)surface->pixels, pixels_size);
  (*env)->SetObjectField(env, sdl_surface_obj, fld_pixels, pixels_obj);
  
  (*env)->SetIntField(env, sdl_surface_obj, fld_pointer, (jint)surface);
  (*env)->SetIntField(env, sdl_surface_obj, fld_flags, surface->flags);
  (*env)->SetIntField(env, sdl_surface_obj, fld_w, surface->w);
  (*env)->SetIntField(env, sdl_surface_obj, fld_h, surface->h);
  (*env)->SetIntField(env, sdl_surface_obj, fld_pitch, surface->pitch);
  (*env)->SetIntField(env, sdl_surface_obj, fld_refcount, surface->refcount);

  return sdl_surface_obj;
}

jobject createObjectFromSDL_VideoInfo(JNIEnv* env, jclass sdl_videoinfo, const SDL_VideoInfo* videoinfo){
  jmethodID cns = (*env)->GetMethodID(env, sdl_videoinfo, "<init>", "()V");
  jobject sdl_videoinfo_obj = (*env)->NewObject(env, sdl_videoinfo, cns);
  
  jclass sdl_format = getSDL_PixelFormat(env);
  jobject sdl_pixel_format_obj = createObjectFromSDL_PixelFormat(env, sdl_format, videoinfo->vfmt);

  jfieldID fld_hw_available = (*env)->GetFieldID(env, sdl_videoinfo, "hw_available", "J");
  jfieldID fld_wm_available = (*env)->GetFieldID(env, sdl_videoinfo, "wm_available", "J");
  jfieldID fld_blit_hw = (*env)->GetFieldID(env, sdl_videoinfo, "blit_hw", "J");
  jfieldID fld_blit_hw_CC = (*env)->GetFieldID(env, sdl_videoinfo, "blit_hw_CC", "J");
  jfieldID fld_blit_hw_A = (*env)->GetFieldID(env, sdl_videoinfo, "blit_hw_A", "J");
  jfieldID fld_blit_sw = (*env)->GetFieldID(env, sdl_videoinfo, "blit_sw", "J");
  jfieldID fld_blit_sw_CC = (*env)->GetFieldID(env, sdl_videoinfo, "blit_sw_CC", "J");
  jfieldID fld_blit_sw_A = (*env)->GetFieldID(env, sdl_videoinfo, "blit_sw_A", "J");
  jfieldID fld_blit_fill = (*env)->GetFieldID(env, sdl_videoinfo, "blit_fill", "J");
  jfieldID fld_video_mem = (*env)->GetFieldID(env, sdl_videoinfo, "video_mem", "J");
  jfieldID fld_vfmt = (*env)->GetFieldID(env, sdl_videoinfo, "vfmt", "Lsdl/PixelFormat;");
  jfieldID fld_current_w = (*env)->GetFieldID(env, sdl_videoinfo, "current_w", "I");
  jfieldID fld_current_h = (*env)->GetFieldID(env, sdl_videoinfo, "current_h", "I");
  
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_hw_available, videoinfo->hw_available);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_wm_available, videoinfo->wm_available);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_blit_hw, videoinfo->blit_hw);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_blit_hw_CC, videoinfo->blit_hw_CC);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_blit_hw_A, videoinfo->blit_hw_A);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_blit_sw, videoinfo->blit_sw);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_blit_sw_CC, videoinfo->blit_sw_CC);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_blit_sw_A, videoinfo->blit_sw_A);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_blit_fill, videoinfo->blit_fill);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_video_mem, videoinfo->video_mem);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_current_w, videoinfo->current_w);
  (*env)->SetIntField(env, sdl_videoinfo_obj, fld_current_h, videoinfo->current_h);
  (*env)->SetObjectField(env, sdl_videoinfo_obj, fld_vfmt, sdl_pixel_format_obj);


  return sdl_videoinfo_obj;
}



/**
 * General
 */
JNIEXPORT jint JNICALL Java_sdl_SDL_init(JNIEnv *env, jobject obj, jint flags){
  return SDL_Init(flags);
}

JNIEXPORT jint JNICALL Java_sdl_SDL_initSubSystem(JNIEnv *env, jobject obj, jint flags){
  return SDL_InitSubSystem(flags);
}

JNIEXPORT void JNICALL Java_sdl_SDL_quitSubSystem(JNIEnv *env, jobject obj, jint flags){
  return SDL_QuitSubSystem(flags);
}

JNIEXPORT void JNICALL Java_sdl_SDL_quit(JNIEnv *env, jobject obj){
  (*env)->DeleteGlobalRef(env,g_surface);
  g_surface = NULL;
  SDL_Quit();
  return;
}

JNIEXPORT jint JNICALL Java_sdl_SDL_wasInit(JNIEnv *env, jobject obj, jint flags){
  return SDL_WasInit(flags);
}

JNIEXPORT jstring JNICALL Java_sdl_SDL_getError(JNIEnv *env, jobject obj){
  char* error = SDL_GetError();
  return (*env)->NewStringUTF(env,error);
}

JNIEXPORT void JNICALL Java_sdl_SDL_setError(JNIEnv *env, jobject obj){
}

JNIEXPORT void JNICALL Java_sdl_SDL_Error(JNIEnv *env, jobject obj){
}

JNIEXPORT void JNICALL Java_sdl_SDL_clearError(JNIEnv *env, jobject obj){
}

JNIEXPORT jint JNICALL Java_sdl_SDL_loadObject(JNIEnv *env, jobject obj, jstring sofile){
}

JNIEXPORT jint JNICALL Java_sdl_SDL_loadFunction(JNIEnv *env, jobject obj, jint handle, jstring name){
}

JNIEXPORT void JNICALL Java_sdl_SDL_unloadObject(JNIEnv *env, jobject obj, jint handle){
}

JNIEXPORT jobject JNICALL Java_sdl_SDL_getVersion(JNIEnv *env, jobject obj){
}

JNIEXPORT jobject JNICALL Java_sdl_SDL_getLinkedVersion(JNIEnv *env, jobject obj){
}


/**
 * Video
 */
JNIEXPORT jobject JNICALL Java_sdl_SDL_getVideoSurface(JNIEnv *env, jobject obj){
  return g_surface; 
}

JNIEXPORT jobject JNICALL Java_sdl_SDL_getVideoInfo(JNIEnv *env, jobject obj){
  const SDL_VideoInfo* videoinfo = SDL_GetVideoInfo();
  jclass cls_videoinfo = getSDL_VideoInfo(env);
  jobject sdl_videoinfo_obj = createObjectFromSDL_VideoInfo(env, cls_videoinfo, videoinfo);
    
  return sdl_videoinfo_obj;
}

JNIEXPORT jstring JNICALL Java_sdl_SDL_videoDriverName(JNIEnv *env, jobject obj){
  char buffer[BUFFER_SIZE];
  jstring driver_name = NULL;
  if(SDL_VideoDriverName(buffer, BUFFER_SIZE) != NULL){
    driver_name = (*env)->NewStringUTF(env, buffer); 
  }

  return driver_name;
}

JNIEXPORT jobject JNICALL Java_sdl_SDL_listModes(JNIEnv *env, jobject obj){
}

JNIEXPORT jint JNICALL Java_sdl_SDL_videoModeOK(JNIEnv *env, jobject obj, jint width, jint height, jint bpp, jlong flags){
}

JNIEXPORT jobject JNICALL Java_sdl_SDL_setVideoMode(JNIEnv *env, jobject obj, jint width, jint height, jint bitsperpixel, jint flags){
  SDL_Surface* surface = SDL_SetVideoMode(width, height, bitsperpixel, flags);
  jclass sdl_surface = getSDL_Surface(env);
  jobject sdl_surface_obj = createObjectFromSDL_Surface(env, sdl_surface, surface);
  g_surface = (*env)->NewGlobalRef(env,sdl_surface_obj);
  return g_surface;
}

JNIEXPORT void JNICALL Java_sdl_SDL_updateRect(JNIEnv *env, jobject obj, jint surface, jint x, jint y, jint w, jint h){
}

JNIEXPORT void JNICALL Java_sdl_SDL_updateRects(JNIEnv *env, jobject obj, jint surface, jint numrects, jobject rects){
}


JNIEXPORT jint JNICALL Java_sdl_SDL_flip(JNIEnv *env, jobject obj, jobject surface){
  jobject sdl_surface_obj = (*env)->GetObjectClass(env, surface);
  jclass sdl_surface = getSDL_Surface(env);
  jfieldID fld_pointer = (*env)->GetFieldID(env, sdl_surface, "pointer", "I");
  jint surface_pointer = (*env)->GetIntField(env, surface, fld_pointer);

  return SDL_Flip((SDL_Surface*)surface_pointer);
}

JNIEXPORT jint JNICALL Java_sdl_SDL_setColors(JNIEnv *env, jobject obj, jobject surface, jobjectArray colors, jint firstcolor, jint ncolors){
  jint surface_pointer = getSDL_SurfacePointer(env, surface);
  jclass color_class = getSDL_Color(env);

  jfieldID fld_r = (*env)->GetFieldID(env, color_class, "r", "I");
  jfieldID fld_g = (*env)->GetFieldID(env, color_class, "g", "I");
  jfieldID fld_b = (*env)->GetFieldID(env, color_class, "b", "I");
  
  jsize size = (*env)->GetArrayLength(env,colors);
  SDL_Color sdl_colors[size];
  int i=0;
  for(i=0; i<size; i++){
    jobject color_obj = (*env)->GetObjectArrayElement(env, colors, i);
    jint color_r = (*env)->GetIntField(env, color_obj, fld_r);
    jint color_g = (*env)->GetIntField(env, color_obj, fld_g);
    jint color_b = (*env)->GetIntField(env, color_obj, fld_b);
    sdl_colors[i].r = color_r;
    sdl_colors[i].g = color_g;
    sdl_colors[i].b = color_b;
  }
  return SDL_SetColors((SDL_Surface*)surface_pointer, sdl_colors, firstcolor, ncolors);
}

JNIEXPORT jint JNICALL Java_sdl_SDL_setPalette(JNIEnv *env, jobject obj, jobject surface, jint flags, jobjectArray colors, jint firstcolor, jint ncolors){
  jint surface_pointer = getSDL_SurfacePointer(env, surface);
  jclass color_class = getSDL_Color(env);

  jfieldID fld_r = (*env)->GetFieldID(env, color_class, "r", "I");
  jfieldID fld_g = (*env)->GetFieldID(env, color_class, "g", "I");
  jfieldID fld_b = (*env)->GetFieldID(env, color_class, "b", "I");

  jobject color_obj = (*env)->GetObjectClass(env, colors);
  jint color_r = (*env)->GetIntField(env, color_obj, fld_r);
  jint color_g = (*env)->GetIntField(env, color_obj, fld_g);
  jint color_b = (*env)->GetIntField(env, color_obj, fld_b);

  jsize size = (*env)->GetArrayLength(env,colors);
  SDL_Color sdl_colors[size];
  int i = 0;
  for(i=0; i<size; i++){
    jobject color_obj = (*env)->GetObjectArrayElement(env, colors, i);
    jint color_r = (*env)->GetIntField(env, color_obj, fld_r);
    jint color_g = (*env)->GetIntField(env, color_obj, fld_g);
    jint color_b = (*env)->GetIntField(env, color_obj, fld_b);
    sdl_colors[i].r = color_r;
    sdl_colors[i].g = color_g;
    sdl_colors[i].b = color_b;
  }

  return SDL_SetPalette((SDL_Surface*)surface_pointer, flags, sdl_colors, firstcolor, ncolors);
}


JNIEXPORT jlong JNICALL Java_sdl_SDL_mapRGB(JNIEnv *env, jobject obj, jobject fmt, jint r, jint g, jint b){
  jobject sdl_pixel_format_obj = (*env)->GetObjectClass(env, fmt);
  jclass sdl_pixel_format = getSDL_PixelFormat(env);
  jfieldID fld_pointer = (*env)->GetFieldID(env, sdl_pixel_format, "pointer", "I");
  jint fmt_pointer = (*env)->GetIntField(env, fmt, fld_pointer);
  return SDL_MapRGB((SDL_PixelFormat*)fmt_pointer, r, g, b);
}


JNIEXPORT jint JNICALL Java_sdl_SDL_lockSurface(JNIEnv *env, jobject obj, jobject surface){
  jobject sdl_surface_obj = (*env)->GetObjectClass(env, surface);
  jclass sdl_surface = getSDL_Surface(env);
  jfieldID fld_pointer = (*env)->GetFieldID(env, sdl_surface, "pointer", "I");
  jint surface_pointer = (*env)->GetIntField(env, surface, fld_pointer);
  return SDL_LockSurface((SDL_Surface*)surface_pointer);
}

JNIEXPORT void JNICALL Java_sdl_SDL_unlockSurface(JNIEnv *env, jobject obj, jobject surface){
  jobject sdl_surface_obj = (*env)->GetObjectClass(env, surface);
  jclass sdl_surface = getSDL_Surface(env);
  jfieldID fld_pointer = (*env)->GetFieldID(env, sdl_surface, "pointer", "I");
  jint surface_pointer = (*env)->GetIntField(env, surface, fld_pointer);
  SDL_UnlockSurface((SDL_Surface*)surface_pointer);
  return;
}


/**
 * Window Management
 */
JNIEXPORT void JNICALL Java_sdl_SDL_setCaption(JNIEnv *env, jobject obj, jstring title, jint icon){
}

JNIEXPORT jlong JNICALL Java_sdl_SDL_getTicks(JNIEnv *env, jobject obj){
  return (jlong)SDL_GetTicks();
}

JNIEXPORT void JNICALL Java_sdl_SDL_delay(JNIEnv *env, jobject obj, jlong ms){
  SDL_Delay((unsigned int)ms);
  return;
}

