/*************************************************
* ScalaSDL
*
*  /|/|
* (- - )
* (    )~
*
* RoNor
* ronor.u@gmail.com
* -----------------------------------------------
* SDL.scala
* SDLクラス及びそのコンパニオンクラスは、
* SDLのネイティブAPIと1対1で対応するメソッドを提供します。
*
* 参考資料
* http://www.libsdl.org/cgi/docwiki.cgi/SDL_API
*************************************************/
package sdl {
  
  object SDL {
    System.loadLibrary("SDL")
    //flags for init
    val INIT_TIMER = 0x00000001
    val INIT_AUDIO = 0x00000010
    val INIT_VIDEO = 0x00000020
    val INIT_CDROM = 0x00000100
    val INIT_JOYSTICK = 0x00000200
    val INIT_NOPARACHUTE = 0x00100000
    val INIT_EVENTTHREAD = 0x01000000
    val INIT_EVERYTHING = 0x0000FFFF

    //flags for sdl_surface
    val SWSURFACE = 0x00000000 //system memory
    val HWSURFACE = 0x00000001 //video memory
    val ASYNCBLIT = 0x00000004 //use async blits
    val ANYFORMAT = 0x10000000
    val HWPALETTE = 0x20000000
    val DOUBLEBUF = 0x40000000
    val FULLSCREEN = 0x80000000
    val OPENGL = 0x00000002
    val OPENGLBLIT = 0x0000000A
    val RESIZABLE = 0x00000010
    val NOFRAME = 0x00000020
    val HWACCEL = 0x00000100
    val SRCCOLORKEY = 0x00001000
    val RLEACCELOK = 0x00002000
    val SRCALPHA = 0x00010000
    val PREALLOC = 0x01000000

    //flags for setPalette()
    val LOGPAL = 0x01
    val PHYSPAL = 0x02
   
    //overlay formats
    val YV12_OVERLAY = 0x32315659
    val IYUV_OVERLAY = 0x56555949
    val YUY2_OVERLAY = 0x32595559
    val YVYU_OVERLAY = 0x55595659

    private val sdl = new SDL()

    //-------------------------------------------------
    // General
    //-------------------------------------------------
    def init(flags:Int):Int = return sdl.init(flags) 
    def initSubSystem(flags:Int):Int = return sdl.initSubSystem(flags)
    def quitSubSystem(flags:Int):Unit = return sdl.quitSubSystem(flags)
    def quit():Unit = return sdl.quit()
    def wasInit(flags:Int):Int = return sdl.wasInit(flags)
    def getError():String = return sdl.getError()
    def setError():Unit = return sdl.setError()
    def error():Unit = return sdl.error()
    def clearError():Unit = return sdl.clearError()
    def loadObject(sofile:String):Int = return sdl.loadObject(sofile)
    def loadFunction(handle:Int, name:String):Int = return sdl.loadFunction(handle, name)
    def unloadObject(handle:Int):Unit = return sdl.unloadObject(handle)
    def getVersion():Version = return sdl.getVersion()
    def getLinkedVersion():Version = return sdl.getLinkedVersion()

    //-------------------------------------------------
    // Video
    //-------------------------------------------------
    def getVideoSurface():Surface = return sdl.getVideoSurface()
    def getVideoInfo():VideoInfo = return sdl.getVideoInfo()
    def videoDriverName():String = return sdl.videoDriverName()
    def listModes(format:PixelFormat, flags:Int):Rect = return sdl.listModes(format, flags)
    def setVideoMode(width:Int, height:Int, bitsperpixel:Int, flags:Int):Surface = return sdl.setVideoMode(width, height, bitsperpixel, flags)
    def flip(surface:Surface):Int = sdl.flip(surface)
    def setColors(surface:Surface, colors:Array[Color], firstcolor:Int, ncolor:Int):Int = return sdl.setColors(surface, colors, firstcolor, ncolor)
    def setPalette(surface:Surface, flags:Int, colors:Array[Color], firstcolor:Int, ncolors:Int):Int = return sdl.setPalette(surface, flags, colors, firstcolor, ncolors)
    def mapRGB(fmt:PixelFormat, r:Int, g:Int, b:Int):Long = return sdl.mapRGB(fmt, r, g, b);
    def lockSurface(surface:Surface):Int = return sdl.lockSurface(surface)
    def unlockSurface(surface:Surface):Unit = return sdl.unlockSurface(surface)
    def blitSurface(surface:Surface, srcrect:Rect, dst:Surface, dstrect:Rect):Int = sdl.blitSurface(surface, srcrect, dst, dstrect)

    //-------------------------------------------------
    // Window Management
    //-------------------------------------------------
    def setCaption(title:String, icon:Int):Unit = return sdl.setCaption(title, icon)
    def getTicks():Long = return sdl.getTicks()
    def delay(ms:Long):Unit = return sdl.delay(ms)
  }

  class SDL {
    //-------------------------------------------------
    // General
    //-------------------------------------------------
    //int SDL_Init(Uint32 flags)
    @native def init(flags:Int):Int
    //int SDL_InitSubSystem(Uint32 flags)
    @native def initSubSystem(flags:Int):Int
    //void SDL_QuitSubSystem(Uint32 flags)
    @native def quitSubSystem(flags:Int):Unit
    //void SDL_Quit()
    @native def quit():Unit
    //int SDL_WasInit(Uint32 flags)
    @native def wasInit(flags:Int):Int
    //char* SDL_GetError()
    @native def getError():String
    //void SDL_SetError(const char *fmt, ...)
    @native def setError():Unit
    //void SDL_Error(SDL_errorcode code)
    @native def error():Unit
    //void SDL_ClearError()
    @native def clearError():Unit
    //void* SDL_LoadObject(const char* sofile)
    @native def loadObject(sofile:String):Int
    //void* SDL_LoadFunction(void* handle, const char* name)
    @native def loadFunction(handle:Int, name:String):Int
    //void SDL_UnloadObject(void* handle)
    @native def unloadObject(handle:Int):Unit
    //コンパイル時のSDLライブラリのバージョンを取得
    @native def getVersion():Version
    //動的にリンクしているSDLライブラリのバージョンを取得
    @native def getLinkedVersion():Version

    //-------------------------------------------------
    // Video
    //-------------------------------------------------
    //SDL_Surface* SDL_GetVideoSurface()
    @native def getVideoSurface():Surface
    //const SDL_VideoInfo* SDL_GetVideoInfo()
    @native def getVideoInfo():VideoInfo
    //char* SDL_VideoDriverName(char* namebuf, int maxlen)
    @native def videoDriverName():String
    //SDL_Rect** SDL_ListModes(SDL_PixelFormat* format, Unit32 flags)
    @native def listModes(format:PixelFormat, flags:Int):Rect
    //int SDL_VideoModeOK(int width, int height, int bpp, Uint32 flags)
    @native def videoModeOK(width:Int, height:Int, bpp:Int, flags:Long):Int
    //SDL_Surface* setVideoMode(int width, int height, int bitsperpixel, Uint32 flags)
    @native def setVideoMode(width:Int, height:Int, bitsperpixel:Int, flags:Int):Surface
    //void SDL_UpdateRect(SDL_Surface* screen, Sint32 x, Sint32 y, Sint32 w, Sint32 h)
    @native def updateRect(screen:Int, x:Int, y:Int, w:Int, h:Int):Unit
    //void SDL_UpdateRects(SDL_Surface* screen, int numrects, SDL_Rect* rects)
    @native def updateRects(screen:Int, numrects:Int, rects:Rect):Unit
    //int SDL_Flip(SDL_Surface* screen)
    @native def flip(screen:Surface):Int
    //int SDL_SetColors(SDL_Surface* surface, SDL_Color *colors, int firstcolor, int ncolor)
    @native def setColors(surface:Surface, colors:Array[Color], firstcolor:Int, ncolor:Int):Int
    //int SDL_SetPalette(SDL_Surface* surface, int flags, SDL_Color* colors, int firstcolor, int ncolors)
    @native def setPalette(surface:Surface, flags:Int, colors:Array[Color], firstcolor:Int, ncolors:Int):Int
    //int SDL_SetGamma(float redgamma, float greengamma, float bluegamma)
    
    //int SDL_GetGammaRamp(Uint16 *redtable, Uint16 *greentable, Uint16 *bluetable)
    
    //int SDL_SetGammaRamp(Uint16 *redtable, Uint16 *greentable, Uint16 *bluetable)

    //Uint32 SDL_MapRGB(SDL_PixelFormat *fmt, Uint8 r, Uint8 g, Uint8 b)
    @native def mapRGB(fmt:PixelFormat, r:Int, g:Int, b:Int):Long
    //Uint32 SDL_MapRGBA(SDL_PixelFormat *fmt, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
    
    //void SDL_GetRGB(Uint32 pixel, SDL_PixelFormat *fmt, Uint8 *r, Uint8 *g, Uint8 *b)
    
    //void SDL_GetRGBA(Uint32 pixel, SDL_PixelFormat *fmt, Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a) 
   
    //SDL_Surface *SDL_CreateRGBSurface(Uint32 flags, int width, int height, int bitsPerPixel, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask)
    
    //SDL_Surface *SDL_CreateRGBSurfaceFrom(void *pixels,int width, int height, int depth, int pitch,Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask)
    
    //void SDL_FreeSurface(SDL_Surface* surface)
    
    //int SDL_LockSurface(SDL_Surface* surface)
    @native def lockSurface(surface:Surface):Int
    //void SDL_UnlockSurface(SDL_Surface* surface)
    @native def unlockSurface(surface:Surface):Unit    
    //SDL_Surface *SDL_ConvertSurface(SDL_Surface *src, SDL_PixelFormat *fmt, Uint32 flags)
    //SDL_Surface *SDL_DisplayFormat(SDL_Surface *surface)
    //SDL_Surface *SDL_DisplayFormatAlpha(SDL_Surface *surface)
    //int SDL_SaveBMP(SDL_Surface *surface, const char *file)
    //int SDL_SetColorKey(SDL_Surface *surface, Uint32 flag, Uint32 key)
    //int SDL_SetAlpha(SDL_Surface *surface, Uint32 flags, Uint8 alpha)
    //void SDL_SetClipRect(SDL_Surface *surface, SDL_Rect *rect)
    //void SDL_GetClipRect(SDL_Surface *surface, SDL_Rect *rect)
    //int SDL_BlitSurface(SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect)
    @native def blitSurface(surface:Surface, srcrect:Rect, dst:Surface, dstrect:Rect):Int
    //int SDL_FillRect(SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color)
    
    //---GL---
    //int SDL_GL_LoadLibrary(const char *path)
    //void *SDL_GL_GetProcAddress(const char* proc)
    //int SDL_GL_GetAttribute(SDL_GLattr attr, int *value)
    //int SDL_GL_SetAttribute(SDL_GLattr attr, int value)
    //void SDL_GL_SwapBuffers(void)
    
    //---YUV---
    //SDL_Overlay *SDL_CreateYUVOverlay(int width, int height, Uint32 format, SDL_Surface *display)
    //int SDL_LockYUVOverlay(SDL_Overlay *overlay)
    //void SDL_UnlockYUVOverlay(SDL_Overlay *overlay)
    //int SDL_DisplayYUVOverlay(SDL_Overlay *overlay, SDL_Rect *dstrect)
    //void SDL_FreeYUVOverlay(SDL_Overlay *overlay)
    
    //-------------------------------------------------
    // Window Management
    //-------------------------------------------------
    //int SDL_GetWMInfo(SDL_SysWMinfo *info)
    //void SDL_WM_SetCaption(const char *title, const char *icon)
    @native def setCaption(title:String, icon:Int):Unit
    //void SDL_WM_GetCaption(char **title, char **icon)
    //void SDL_WM_SetIcon(SDL_Surface *icon, Uint8 *mask)
    //int SDL_WM_IconifyWindow(void)
    //int SDL_WM_ToggleFullScreen(SDL_Surface *surface)
    //SDL_GrabMode SDL_WM_GrabInput(SDL_GrabMode mode)
    
    //-------------------------------------------------
    // Events
    //-------------------------------------------------
    
    //-------------------------------------------------
    // Mouse
    //-------------------------------------------------

    //-------------------------------------------------
    // Joystick
    //-------------------------------------------------
    
    //-------------------------------------------------
    // Audio
    //-------------------------------------------------
    
    //-------------------------------------------------
    // CD-ROM
    //-------------------------------------------------
    
    //-------------------------------------------------
    // Multi-threaded programming
    //-------------------------------------------------

    //-------------------------------------------------
    // TIME
    //-------------------------------------------------
    //Uint32 SDL_GetTicks(void)
    @native def getTicks():Long
    //void SDL_Delay(Uint32 ms)
    @native def delay(ms:Long):Unit
    //SDL_TimerID SDL_AddTimer(Uint32 interval, SDL_NewTimerCallback callback, void* param)
    //SDL_bool SDL_RemoveTimer(SDL_TimerID id)
    //int SDL_SetTimer(Uint32 interval, SDL_TimerCallback callback)
    
    //-------------------------------------------------
    // Files
    //-------------------------------------------------
    //SDL_RWops *SDL_RWFromFile(const char *file, const char *mode)
    //SDL_RWops *SDL_RWFromFP(FILE *fp, int autoclose)
    //SDL_RWops *SDL_RWFromMem(void *mem, int size)
    //SDL_RWops *SDL_RWFromConstMem(const void *mem, int size)
    //SDL_RWops *SDL_AllocRW(void)
    //void SDL_FreeRW(SDL_RWops *context)
    //#define SDL_RWseek(ctx, offset, whence) (ctx)->seek(ctx, offset, whence)
    //#define SDL_RWtell(ctx)                 (ctx)->seek(ctx, 0, RW_SEEK_CUR)
    //#define SDL_RWread(ctx, ptr, size, n)   (ctx)->read(ctx, ptr, size, n)
    //#define SDL_RWwrite(ctx, ptr, size, n)  (ctx)->write(ctx, ptr, size, n)
    //#define SDL_RWclose(ctx)                (ctx)->close(ctx)
  }
}
